from invenio_records_resources.services import RecordService
from invenio_records_resources.services import RecordService as InvenioRecordService


class CommonService(RecordService):
    """CommonRecord service."""
